package org.nisum.service;

import org.nisum.model.Notification;
import org.nisum.producer.NotificationProducer;
import org.nisum.repository.NotificationRepository;
import org.nisum.webclientconfig.WebClientConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;

@Service
public class NotificationService {

    @Autowired
    private final NotificationProducer notificationProducer;
    @Autowired
    private final NotificationRepository notificationRepository;


    private final WebClient webClient;


    public NotificationService(NotificationProducer notificationProducer, NotificationRepository notificationRepository,WebClient.Builder webClientBuilder) {
        this.notificationProducer = notificationProducer;
        this.notificationRepository = notificationRepository;
        this.webClient = webClientBuilder.baseUrl("https://jsonplaceholder.typicode.com").build();
    }

    public void sendNotification(String userId, String message){
        notificationProducer.sendNotification(message);
        Notification notification = new Notification();
        notification.setUserId(userId);
        notification.setMessage(message);
        notification.setRead(false);
        notification.setTimestamp(LocalDateTime.now());

        notificationRepository.save(notification).subscribe();
    }

    public Mono<String> sendNotification(){
            return this.webClient.get()
                    .uri("/posts")
                    .retrieve()
                    .bodyToMono(String.class);
    }

    public Flux<Notification> getNotificationForUser(String userId){
        return notificationRepository.findByUserId(userId);
    }
}
